@@ -5,9 +5,9 @@ import (
 	"strconv"
 	"strings"
 
-	"github.com/sagernet/sing-shadowsocks"
 	"github.com/sagernet/sing-shadowsocks/shadowaead_2022"
 	C "github.com/sagernet/sing/common"
+	A "github.com/sagernet/sing/common/auth"
 	B "github.com/sagernet/sing/common/buf"
 	"github.com/sagernet/sing/common/bufio"
 	E "github.com/sagernet/sing/common/exceptions"
@@ -119,14 +119,13 @@ func (i *RelayInbound) Process(ctx context.Context, network net.Network, connect
 }
 
 func (i *RelayInbound) NewConnection(ctx context.Context, conn net.Conn, metadata M.Metadata) error {
-	userCtx := ctx.(*shadowsocks.UserContext[int])
 	inbound := session.InboundFromContext(ctx)
-	user := i.destinations[userCtx.User]
+	user, _ := A.UserFromContext[User](ctx)
 	inbound.User = &protocol.MemoryUser{
 		Email: user.Email,
 		Level: uint32(user.Level),
 	}
-	ctx = log.ContextWithAccessMessage(userCtx.Context, &log.AccessMessage{
+	ctx = log.ContextWithAccessMessage(ctx, &log.AccessMessage{
 		From:   metadata.Source,
 		To:     metadata.Destination,
 		Status: log.AccessAccepted,
@@ -147,14 +146,13 @@ func (i *RelayInbound) NewConnection(ctx context.Context, conn net.Conn, metadat
 }
 
 func (i *RelayInbound) NewPacketConnection(ctx context.Context, conn N.PacketConn, metadata M.Metadata) error {
-	userCtx := ctx.(*shadowsocks.UserContext[int])
 	inbound := session.InboundFromContext(ctx)
-	user := i.destinations[userCtx.User]
+	user, _ := A.UserFromContext[User](ctx)
 	inbound.User = &protocol.MemoryUser{
 		Email: user.Email,
 		Level: uint32(user.Level),
 	}
-	ctx = log.ContextWithAccessMessage(userCtx.Context, &log.AccessMessage{
+	ctx = log.ContextWithAccessMessage(ctx, &log.AccessMessage{
 		From:   metadata.Source,
 		To:     metadata.Destination,
 		Status: log.AccessAccepted,
