@@ -15,7 +15,7 @@ import (
 //go:generate go run github.com/xtls/xray-core/common/errors/errorgen
 
 func toDestination(socksaddr M.Socksaddr, network net.Network) net.Destination {
-	if socksaddr.Family().IsFqdn() {
+	if socksaddr.IsFqdn() {
 		return net.Destination{
 			Network: network,
 			Address: net.DomainAddress(socksaddr.Fqdn),
